#validate number
def valid_num(n):
    if isinstance(n,int):
        return True
    else:
        return False

#validate scale
def valid_scale(c):
    if (c=="i" or c=="m"):
        return True
    else: 
        return False

#BMI scale
def BMI_scale(o):
    if (o<18.5):
        return "You are underweight"
    elif (18.5<=o<25):
        return "You are healthy"
    elif (25 <=o<30):
        return "You are overweight"
    elif (30<=o):
        return "You are obese"
    
#staticmain
def howfat(age, height, weight, scale, error=0) :
    '''
    Returns the Body Mass Index (BMI) and its corresponding description on the weight scale.
    Arguments taken are age, height, weight and scale.
    - Valid integer values should be passed for age, height and weight or will return an error.
    - For both imperial and metric, all height measurements must be rounded to the nearest integer value (the deviation from rounding can be considered miniscule in calculating BMI)
    - For imperial scale, height argument must be passed in inches. (i.e. 5ft=60)
    - Scale argument must be defined by a single string: "m" (metric) or "i" (imperial).
    BMI calculation formula derived from:
    imperial: https://www.cdc.gov/nccdphp/dnpao/growthcharts/training/bmiage/page5_2.html#:~:text=When%20using%20English%20measurements%2C%20ounces,a%20conversion%20factor%20of%20703.
    metric: https://www.cdc.gov/nccdphp/dnpao/growthcharts/training/bmiage/page5_1.html
    BMI to Obesity Scale is from:
    https://www.cdc.gov/obesity/basics/adult-defining.html
    '''
    if (valid_num(age)==True and valid_num(height)==True and valid_num(weight)==True and valid_scale(scale)==True) :
        if (18<=age<=65):
            if (scale == "i"):
                BMI = round((weight/height/height)*703,2)
                obesity = BMI_scale(BMI)
                return BMI,obesity
            elif (scale == "m"):
                BMI = round((weight/height/height)*10000,2)
                obesity = BMI_scale(BMI)
                return BMI,obesity
        else:
            if (scale == "i"):
                error_msg = "Please note that BMI for children, teens and seniors may not be accurate."
                BMI = round((weight/height/height)*703,2)
                obesity = BMI_scale(BMI)
                return error_msg,BMI,obesity
            elif (scale == "m"):
                error_msg = "Please note that BMI for children, teens and seniors may not be accurate."
                BMI = round((weight/height/height)*10000,2)
                obesity = BMI_scale(BMI)
                return error_msg,BMI,obesity
    else:
        raise Exception('Please check age, height, weight are integer values and scale is "i" or "m"')




import math

#--------------------------------convertScale()----------------------------
#Convert from imperial to metric
def convertScale(height,weight):
    #Invalid height and weight entries
    if type(height)!=float and type(height)!=int:
        return -1,-1
    if type(weight)!=float and type(weight)!=int:
        return -1,-1
    #height and weight boundaries
    if height<1 or height>107:
        return -1,-1
    if weight<1 or weight>1400:
        return -1,-1
    height=float(round(height*2.54,2))
    weight=float(round(weight*0.45359237,8))
    return height,weight


#------------------------------------calories()--------------------------------------
'''activityLevel 
    *0-Basal Metabolic Rate
    *1-Sedentary: little or no exercise
    *2-Lightly active: exercise 1-3 times a week
    *3-Moderately active: exercise 3-5 times a week
    *4-Active: exercise 6-7 times a week
    *5-Very active: hard exercise 6-7 times a week
'''
#Calculates the user's recommended daily calorie intake based on their AMR
def calories(age,gender,height,weight,activityLevel,scale):
    #^^^^^^^^^^INVALID INPUT CATCHES^^^^^^^^^^
    #Invalid age type
    if type(age)!=int:
        return -1
    #Invalid age
    if age<1 or age>120:
        return -1
    #Invalid gender entry
    if gender!="f" and gender!="m":
        return -1
    #Invalid height and weight entries
    if type(height)!=float and type(height)!=int:
        return -1
    if type(weight)!=float and type(weight)!=int:
        return -1
    #Invalid scale
    if scale!="i" and scale!="m":
        return -1
    #height and weight restrictions
    if scale=="i":
        if height<1 or height>107:
            return -1 
        if weight<1 or weight>1400:
            return -1
    if scale=="m":
        if height<1 or height>272:
            return -1 
        if weight<1 or weight>635:
            return -1 
    #Invalid activity level type
    if type(activityLevel)!=int:
        return -1
    #Invalid activity level range
    if activityLevel<0 or activityLevel>5:
        return -1
   
   
    
    #^^^^^^^^^^CALCULATIONS^^^^^^^^^^
    #******* Convert scale *******  
    if scale=="i":
        height,weight=convertScale(height,weight)

    #******* First calculate the user's BMR *******
    #Calculation for a female
    if gender=="f":
        bmr=655.1 + (9.563*weight) + (1.850*height)-(4.676*age)
    #Calculation for a male
    elif gender=="m":
        bmr=66.47+(13.75*weight)+(5.003*height)-(6.755*age)
       
    
    #******* Calculate by activity level *******
    if activityLevel==0:
        amr=bmr
    elif activityLevel==1:
        amr= bmr*1.2
    elif activityLevel==2:
        amr= bmr*1.375
    elif activityLevel==3:
        amr= bmr*1.55
    elif activityLevel==4:
        amr= bmr*1.725
    elif activityLevel==5:
        amr= bmr*1.9
    
    # return AMR
    return math.ceil(amr)


    




# https://healthyeater.com/how-to-calculate-your-macros
def validDimensions(height=1, age=1, weight=1):
    if not 1<=age<=120:
        raise Exception("Invalid age parameter. Please enter an age between 1 and 120, inclusive")
    if not 1<=height<=272:
        raise Exception("Invalid height parameter. Please enter a height between 1 and 107, inclusive")
    if not 1<=weight<=1400:
        raise Exception("Invalid weight parameter. Please enter a height between 1 and 1400, inclusive")
    return True

def calculateREE(age, gender, height, weight, scale):
    '''
    calculates Resting Energy Expenditure (REE) using Mifflin-St Jeor formula
    '''
    validDimensions(height, age, weight)
    if scale == 'i':
        #converting lbs to kgs and in to cm
        weight = weight / 2.205
        height = height * 2.54
    elif scale != 'm':
        raise Exception("Invalid scale parameter. Please enter either \
            'i' for imperial or 'm' for metric")
    # male version
    if gender == 'm':
        return 10 * weight + 6.25 * height - 5 * age + 5
    # female version
    elif gender == 'f':
        return 10 * 6.25 * height - 5 * age - 161
    else:
        raise Exception("invalid gender parameter. Input \
            'm' for male and 'f' for female.")

def calculateTDEE(REE, userActivityLevel=2):
    '''
    calculates Total Daily Energy Expenditure (TDEE) in 
    calories for a given REE and self-reported activity level
    from sedentary (1) to very active (4).
    
    - Consuming more than your TDEE = weight gain
    - Consuming less than your TDEE = weight loss
    - Consuming the same as your TDEE = weight maintenance
    '''
    activityLevelDict = {
        1: REE * 1.2,
        2: REE * 1.375,
        3: REE * 1.55,
        4: REE * 1.725
    }
    try:
        return activityLevelDict[userActivityLevel]
    except KeyError:
        raise Exception("Invalid Activity Level parameter. \
            Please enter a number from 1 (sedentary) to 4 (very active))")
def weightLossOrGainCalculator(TDEE, lossOrGain='l'):
    '''
    calculates daily calorie consumption needed to lose or gain based on a user's TDEE

    lossOrGain parameter is either 'l' or 'g' and indicates whether the user \
    intends to lose or gain weight(i.e., build muscle)
    '''
    if lossOrGain != 'l' and lossOrGain != 'g':
        raise Exception("Invalid Loss or Gain parameter. Please enter either \
        'l' for loss or 'g' for gain")
    targetTDEE = TDEE - .2*TDEE if lossOrGain == 'l' else TDEE + .2*TDEE
    return targetTDEE

def macros(weight, targetTDEE, scale):
    validDimensions(weight=weight)
    if scale == 'm':
        weight = weight * 2.205
    elif scale != 'i':
        raise Exception("Invalid scale parameter. Please enter either \
            'i' for imperial or 'm' for metric")
    '''
    calculate ideal macronutrient ratios for a given weight and TDEE
    '''
    dailyProteinInGrams = .825 * weight
    dailyProteinInCalories = dailyProteinInGrams * 4

    dailyFatInCalories = .25 * targetTDEE
    dailyFatInGrams = dailyFatInCalories / 9

    dailyCarbsInCalories = targetTDEE - dailyFatInCalories - dailyProteinInCalories
    dailyCarbsInGrams = dailyCarbsInCalories / 4

    # return macro proportions in grams 
    macroDict = {
        'Protein Intake': dailyProteinInGrams,
        'Fat Intake': dailyFatInGrams,
        'Carb Intake': dailyCarbsInGrams
    }

    return macroDict
